import java.awt.Color;

public final class ColorVector
{
	public static final ColorVector RED     = new ColorVector(Color.RED);
	public static final ColorVector GREEN   = new ColorVector(Color.GREEN);
	public static final ColorVector YELLOW  = new ColorVector(Color.YELLOW);
	public static final ColorVector MAGENTA = new ColorVector(Color.MAGENTA);
	public static final ColorVector ORANGE  = new ColorVector(Color.ORANGE);
	public static final ColorVector BLUE    = new ColorVector(Color.BLUE);
	public static final ColorVector CYAN    = new ColorVector(Color.CYAN);
	public static final ColorVector PINK    = new ColorVector(Color.PINK);
	public static final ColorVector WHITE   = new ColorVector(Color.WHITE);
	public static final ColorVector GRAY    = new ColorVector(Color.GRAY);;
	public static final ColorVector BLACK   = new ColorVector(Color.BLACK);;

	private float R, G, B;

	public ColorVector(Color AWTColor)
	{
		R = AWTColor.getRed();
		G = AWTColor.getGreen();
		B = AWTColor.getBlue();
	}

	public ColorVector(ColorVector ExistingColor)
	{
		R = ExistingColor.GetR();
		G = ExistingColor.GetG();
		B = ExistingColor.GetB();
	}
	
	public ColorVector(int RGB)
	{
		R = ((RGB & 0xFF0000) >> 16);
		G = ((RGB & 0x00FF00) >> 8);
		B = (RGB & 0x0000FF);
	}
	
	public ColorVector(float R, float G, float B)
	{
		this.R = R;
		this.G = G;
		this.B = B;
	}

	public float GetR()
	{
		return R;
	}

	public float GetG()
	{
		return G;
	}

	public float GetB()
	{
		return B;
	}
	
	public ColorVector ClampColor()
	{
		R = GetClampedR();
		G = GetClampedG();
		B = GetClampedB();
		
		return this;
	}

	public int GetClampedR()
	{
		return (R > 255) ? 255 : (int)R;
	}

	public int GetClampedG()
	{
		return (G > 255) ? 255 : (int)G;
	}

	public int GetClampedB()
	{
		return (B > 255) ? 255 : (int)B;
	}
	
	public int GetClampedRGB()
	{
		return ((GetClampedR() << 16) | (GetClampedG() << 8) | GetClampedB());
	}
	
	public ColorVector Add(ColorVector V)
	{
		R += V.GetR();
		G += V.GetG();
		B += V.GetB();

		return this;
	}
	
	public ColorVector Add(float cR, float cG, float cB)
	{
		R += cR;
		G += cG;
		B += cB;

		return this;
	}

	public ColorVector Add(float Scalar)
	{
		R += Scalar;
		G += Scalar;
		B += Scalar;

		return this;
	}
	
	public ColorVector Subtract(ColorVector V)
	{
		R -= V.GetR();
		G -= V.GetG();
		B -= V.GetB();

		return this;
	}
	
	public ColorVector Subtract(float cR, float cG, float cB)
	{
		R -= cR;
		G -= cG;
		B -= cB;

		return this;
	}

	public ColorVector Subtract(float Scalar)
	{
		R -= Scalar;
		G -= Scalar;
		B -= Scalar;

		return this;
	}
	
	public ColorVector Multiply(ColorVector V)
	{
		R *= (V.GetClampedR() / 255.0f);
		G *= (V.GetClampedG() / 255.0f);
		B *= (V.GetClampedB() / 255.0f);

		return this;
	}
	
	public ColorVector Multiply(float cR, float cG, float cB)
	{
		R *= cR;
		G *= cG;
		B *= cB;

		return this;
	}

	public ColorVector Multiply(float Scalar)
	{
		R *= Scalar;
		G *= Scalar;
		B *= Scalar;

		return this;
	}
	
	public ColorVector Divide(ColorVector V)
	{
		R /= (V.GetClampedR() / 255.0f);
		G /= (V.GetClampedG() / 255.0f);
		B /= (V.GetClampedB() / 255.0f);

		return this;
	}
	
	public ColorVector Divide(float cR, float cG, float cB)
	{
		R /= cR;
		G /= cG;
		B /= cB;

		return this;
	}

	public ColorVector Divide(float Divide)
	{
		R /= Divide;
		G /= Divide;
		B /= Divide;

		return this;
	}
	
	public static ColorVector Add(ColorVector C1, float cR, float cG, float cB)
	{
		return new ColorVector(C1).Add(cR, cG, cB);
	}

	public static ColorVector Add(ColorVector C1, float Scalar)
	{
		return new ColorVector(C1).Add(Scalar);
	}

	public static ColorVector Add(ColorVector C1, ColorVector C2)
	{
		return new ColorVector(C1).Add(C2);
	}
	
	public static ColorVector Subtract(ColorVector C1, float cR, float cG, float cB)
	{
		return new ColorVector(C1).Subtract(cR, cG, cB);
	}
	
	public static ColorVector Subtract(ColorVector C1, float Scalar)
	{
		return new ColorVector(C1).Subtract(Scalar);
	}

	public static ColorVector Subtract(ColorVector C1, ColorVector C2)
	{
		return new ColorVector(C1).Subtract(C2);
	}

	public static ColorVector Multiply(ColorVector C1, float cR, float cG, float cB)
	{
		return new ColorVector(C1).Multiply(cR, cG, cB);
	}

	public static ColorVector Multiply(ColorVector C1, float Scalar)
	{
		return new ColorVector(C1).Multiply(Scalar);
	}

	public static ColorVector Multiply(ColorVector C1, ColorVector C2)
	{
		return new ColorVector(C1).Multiply(C2);
	}
	
	public static ColorVector Divide(ColorVector C1, float cR, float cG, float cB)
	{
		return new ColorVector(C1).Divide(cR, cG, cB);
	}

	public static ColorVector Divide(ColorVector C1, float Scalar)
	{
		return new ColorVector(C1).Divide(Scalar);
	}

	public static ColorVector Divide(ColorVector C1, ColorVector C2)
	{
		return new ColorVector(C1).Divide(C2);
	}
}