/*
	         Java RayTracer
	     By Dean Camera, 2008
	http://www.fourwalledcubicle.com
*/

public class LightSpotlight implements ILightingObject
{
	private ColorVector LightColor;
	private Vector3D Location;
	private Vector3D Direction;
	private float CosCutoffAngle;

	public LightSpotlight(ColorVector LightColor, Vector3D Location, Vector3D Direction, float CutoffAngle)
	{
		this.Location = Location;
		this.Direction = Direction.UnitVector();
		this.LightColor = LightColor;
		this.CosCutoffAngle = (float)Math.cos(CutoffAngle);
	}

	public Vector3D GetLocation()
	{
		return Location;
	}

	public ColorVector GetLightingDetails(CollisionDetails CollisionRay, float Distance)
	{
		Vector3D NormalToIncident = CollisionRay.GetCollisionNormal();

		float SpotlightComponent = Vector3D.DotProduct(Direction, NormalToIncident);

		// If spotlight is facing more than 90 degrees from the normal, it contributes no light
		if (SpotlightComponent < 0)
		  return ColorVector.BLACK;

		Vector3D PointAtIncidence    = CollisionRay.GetCollisionPoint();
		Vector3D LightToObject       = Vector3D.Subtract(PointAtIncidence, Location);
		Vector3D ObjectToViewer      = Vector3D.Subtract(PointAtIncidence, Constants.EYE_LOCATION);
		Vector3D ReflectRayDirection = LightPhysics.GetReflectionDirection(NormalToIncident, LightToObject);
		
		IMaterial ObjectMaterial = CollisionRay.GetCollisionObject().GetMaterial();
		
		float DiffuseComponent   = ObjectMaterial.GetDiffuse()  * Vector3D.DotProduct(NormalToIncident, LightToObject.UnitVector());
		float SpecularComponent  = ObjectMaterial.GetSpecular() * (float)Math.pow(Vector3D.DotProduct(ObjectToViewer.UnitVector(), ReflectRayDirection), Constants.SPECULAR_ADJUST_POWER);

		ColorVector FinalColor = new ColorVector(0, 0, 0);
		
		if (DiffuseComponent > 0)
			FinalColor.Add(ColorVector.Multiply(LightColor, (DiffuseComponent * Constants.DIFFUSE_ADJUST)));

		if (SpecularComponent > 0)
			FinalColor.Add(ColorVector.Multiply(LightColor, (SpecularComponent * Constants.SPECULAR_ADJUST)));
		
		if (SpotlightComponent < CosCutoffAngle)
			FinalColor.Multiply(SpotlightComponent);

		return FinalColor;
	}
}