/*
	         Java RayTracer
	     By Dean Camera, 2008
	http://www.fourwalledcubicle.com
*/

import java.applet.*;
import java.awt.*;
import java.awt.image.*;
import java.awt.Color;

public final class RayTracer extends Applet implements Runnable
{
	private static  RenderThread[] RenderObjects;
	private static  Thread[]       RenderThreads;
	private int[][] ThreadPixels;

	private static  Thread RenderThread;

	private static MemoryImageSource RenderImageSource;
	private static Image             RenderImage;
	private static int[]             PixelArray;

	public void init()
	{
		System.out.println("INIT - Building Pixel Arrays and Thread Objects...");

		// Create the pixel array which the resulting image will be computed into
		PixelArray = new int[Constants.SCREEN_WIDTH * Constants.SCREEN_HEIGHT];
	
		// Set up the applet dimemsions and visual attributes
		resize(Constants.SCREEN_WIDTH, Constants.SCREEN_HEIGHT);
		setBackground(Color.BLACK);
		
		// Create the an image from the rendered image object from the pixel array
		RenderImageSource = new MemoryImageSource(Constants.SCREEN_WIDTH, Constants.SCREEN_HEIGHT, PixelArray, 0, Constants.SCREEN_WIDTH);
		RenderImageSource.setAnimated(true);
		RenderImage = createImage(RenderImageSource);

		// Create new rendering thread object and Thread arrays
		RenderObjects = new RenderThread[Runtime.getRuntime().availableProcessors()];
		RenderThreads = new Thread[RenderObjects.length];

		ThreadPixels = new int[RenderThreads.length][];

		// Create a thread to manage the rendering - manages the rendering threads
		RenderThread = new Thread(this, "Render Thread");

		System.out.println("INIT - Scene Name:                 " + Constants.SCENE_TO_RENDER.GetSceneName());
		System.out.println("INIT - Scene Objects:              " + Constants.SCENE_TO_RENDER.GetSceneObjects().length);
		System.out.println("INIT - Scene Lights:               " + Constants.SCENE_TO_RENDER.GetSceneLights().length);
		System.out.println("INIT - Scene Supersampling:        " + Constants.SUPER_SAMPLE + " Neighboring Pixels");
		System.out.println("INIT - Scene Refraction Index:     " + Constants.SCENE_REFRACT_INDEX);
		System.out.println("INIT - Scene Recursion Level:      " + Constants.RECURSION_LEVEL);
		System.out.println("INIT - Lines to Render per Thread: " + Constants.RENDER_LINES_PER_THREAD);

		System.out.println("INIT - Starting Render Dispatch Thread...");	
	}

	public void start()
	{
		try
		{
			// Start the rendering dispatch thread
			RenderThread.start();
		}
		catch (Exception E)
		{
		
		}
	}

	public void update(Graphics g)
	{
		// Draw the ray traced image to the JFrame
		g.drawImage(RenderImage, 0, 0, this);
	}

	public void paint(Graphics g)
	{
		// Update the display
		update(g);
	}
	
	public void run()
	{
		int StartLineToRender = 0;

		System.out.println("RENDER - Creating Subrender Thread Pixel Arrays...");

		ThreadPixels = new int[RenderThreads.length][];

		System.out.println("RENDER - Creating Subrender Thread Objects...");

		// Create the rendering thread objects and Threads
		for (int i = 0; i < RenderThreads.length; i++)
		{
			ThreadPixels[i] = new int[Constants.SCREEN_WIDTH * Constants.RENDER_LINES_PER_THREAD];
			RenderObjects[i] = new RenderThread(ThreadPixels[i]);
			RenderThreads[i] = new Thread(RenderObjects[i], "Render Thread " + i);
		}

		System.out.println("RENDER - Starting Render.");

		do
		{
			// Check each rendering thread
			for (int i = 0; i < RenderThreads.length; i++)
			{
				// If all the lines have been processed, early abort
				if (StartLineToRender >= Constants.SCREEN_HEIGHT)
					break;
				
				// Check if thread has finished processing
				if (RenderThreads[i].isAlive() == false)
				{
					// Copy the rendered pixels from the thread's pixel array to the screen pixel array
					System.arraycopy(RenderObjects[i].GetRenderPixels(), 0, PixelArray,
					                 (RenderObjects[i].GetRenderStartLine() * Constants.SCREEN_WIDTH), ThreadPixels[i].length);
				
					// Set up a new thread for the next few lines to render
					RenderObjects[i].SetRenderStartLine(StartLineToRender);
					RenderThreads[i] = new Thread(RenderObjects[i]);
					RenderThreads[i].start();

					// Keep track of how many lines have been processed so far
					StartLineToRender += Constants.RENDER_LINES_PER_THREAD;					

					// Recreate the rendered image from the updated pixel array
					RenderImageSource.newPixels();
				}
			}
			
			Thread.yield();
		}
		while (StartLineToRender < Constants.SCREEN_HEIGHT);
		
		// Cleanup - wait for all final render threads to complete
		for (int i = 0; i < RenderThreads.length; i++)
		{
			try
			{
				// Join current render thread to this thread - blocks until thread is complete
				RenderThreads[i].join();

				// Copy the rendered pixels from the thread's pixel array to the screen pixel array
				System.arraycopy(RenderObjects[i].GetRenderPixels(), 0, PixelArray,
				                 (RenderObjects[i].GetRenderStartLine() * Constants.SCREEN_WIDTH), ThreadPixels[i].length);
			}
			catch (Exception E)
			{
			
			}
		}
		
		// Recreate the rendered image from the updated pixel array
		RenderImageSource.newPixels();
		
		System.out.println("RENDER - Render Complete.");
	}
}
