/*
	         Java RayTracer
	     By Dean Camera, 2008
	http://www.fourwalledcubicle.com
*/

import java.awt.image.*;

public class RenderThread implements Runnable
{
	private int StartLine;
	private int[] PixelData;

	public RenderThread(int[] PixelArray)
	{
		PixelData = PixelArray;
	}
	
	public void SetRenderStartLine(int StartLine)
	{
		this.StartLine = StartLine;
	}

	public int[] GetRenderPixels()
	{
		return PixelData;
	}
	
	public int GetRenderStartLine()
	{
		return StartLine;
	}
	
	public void run()
	{
		int CurrentPixelArrayItem = 0;

		// Loop through each Y pixel on the screen
		for (int YTrace = StartLine; YTrace < (StartLine + Constants.RENDER_LINES_PER_THREAD); ++YTrace)
		{
			// Loop through each X pixel on the screen
			for (int XTrace = 0; XTrace < Constants.SCREEN_WIDTH; ++XTrace)
			{
				// Compute primary ray direction
				Vector3D RayDirection = new Vector3D(XTrace, YTrace, 0).Subtract(Constants.EYE_LOCATION);
				
				int TotalSamples = 0;
				
				ColorVector FinalColor = new ColorVector(0, 0, 0);

				// Retrieve samples near the intersection point
				for (int dX = -Constants.SUPER_SAMPLE; dX <= Constants.SUPER_SAMPLE; ++dX)
				{
					for (int dY = -Constants.SUPER_SAMPLE; dY <= Constants.SUPER_SAMPLE; ++dY)
					{
						Vector3D SuperOrigin = new Vector3D(Constants.EYE_LOCATION).Add(dX * Constants.SUPER_SAMPLE_DELTA, dY * Constants.SUPER_SAMPLE_DELTA, 0);
						Ray SuperRay = new Ray(SuperOrigin, RayDirection);
						
						// Raytrace the current pixel
						FinalColor.Add(SuperRay.TraceRay(Constants.RECURSION_LEVEL + 1));
						
						// Increment the number of samples taken
						TotalSamples++;
					}
				}

				// Average sum of samples to give final super sampled pixel color
				FinalColor.Divide(TotalSamples);

				PixelData[CurrentPixelArrayItem++] = ((255 << 24) | (FinalColor.GetClampedRGB()));
			}
		}
	}
}